<?php

namespace Drupal\user_annex\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines the user_annex entity.
 *
 * While user_annex entities are basically content entities, they do not
 * have a published/unpublished status, nor can they be changed once created.
 * There is no provision for adding, editing, or deleting them through the
 * standard entity api.
 *
 *
 *  "default" = "Drupal\user_annex\Form\UserAnnexForm",
 *
 *
 * @ingroup user_annex
 *
 * @ContentEntityType(
 *   id = "user_annex",
 *   label = @Translation("User annex"),
 *   handlers = {
 *     "view_builder" = "Drupal\user_annex\UserAnnexViewBuilder",
 *     "list_builder" = "Drupal\user_annex\UserAnnexListBuilder",
 *     "views_data" = "Drupal\user_annex\UserAnnexViewsData",
 *
 *     "form" = {
 *       "add" = "Drupal\user_annex\Form\UserAnnexAddForm",
 *       "edit" = "Drupal\user_annex\Form\UserAnnexEditForm",
 *       "delete" = "Drupal\user_annex\Form\UserAnnexDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\user_annex\UserAnnexHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\user_annex\UserAnnexAccessControlHandler",
 *   },
 *   base_table = "user_annex",
 *   translatable = FALSE,
 *   admin_permission = "administer user_annex entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *   },
 *   links = {
 *     "canonical" = "/user_annex/{user_annex}",
 *     "add-form" = "/user_annex/add",
 *     "edit-form" = "/user_annex/{user_annex}/edit",
 *     "delete-form" = "/user_annex/{user_annex}/delete",
 *     "collection" = "/user_annex/list",
 *   },
 *   field_ui_base_route = "user_annex.settings"
 * )
 */
class UserAnnex extends ContentEntityBase implements UserAnnexInterface {

  /**
   * {@inheritdoc}
   *
   * When a new entity instance is added, set the user_id entity reference to
   * the current user as the creator of the instance.
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += array(
      'created' => time(),
      'modified_by' => \Drupal::currentUser()->id(),
    );
  }

  /**
   * Sets the label of the user_annex entity.
   *
   * @param string $label
   *   The label for the user_annex entity.
   *
   * @return \Drupal\user_annex\Entity\UserAnnexInterface
   *   The called user_annex entity.
   */
  public function setLabel($label) {
    $this->set('label', $label);
    return $this;
  }

  /**
   * Gets the creation time of the user_annex entity.
   *
   * @return integer
   *   Creation timestamp of the user_annex entity.
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * Sets the creation time of the user_annex entity.
   *
   * @param integer $timestamp
   *   The creation timestamp for the user_annex entity.
   *
   * @return \Drupal\user_annex\Entity\UserAnnexInterface
   *   The called user_annex entity.
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * Gets the action used to modify the user entity.
   *
   * @return string
   *   Action used to modify the user entity.
   */
  public function getAction() {
    return $this->get('action')->value;
  }

  /**
   * Sets the action used to modify the user entity.
   *
   * @param string $action
   *   The action used to modify the user entity.
   *
   * @return \Drupal\user_annex\Entity\UserAnnexInterface
   *   The called user_annex entity.
   */
  public function setAction($action) {
    $this->set('action', $action);
    return $this;
  }

  /**
   * Gets the User account responsible for modifying the user entity.
   *
   * @return \Drupal\user\UserInterface
   *   User account responsible for user entity change.
   */
  public function getModifiedBy() {
    return $this->get('modified_by')->target_id;
  }

  /**
   * Sets the user account responsible for modifying the user entity.
   *
   * @param \Drupal\user\UserInterface $user
   *   The account responsible for modifying the user entity.
   *
   * @return \Drupal\user_annex\Entity\UserAnnexInterface
   *   The called user_annex entity.
   */
  public function setModifiedBy($user) {
    $this->set('modified_by', $user->id());
    return $this;
  }

  /**
   * Sets the user account responsible for modifying the user entity.
   *
   * @param integer $uid
   *   The id of the account responsible for modifying the user entity.
   *
   * @return \Drupal\user_annex\Entity\UserAnnexInterface
   *   The called user_annex entity.
   */
  public function setModifiedByUid($uid) {
    $this->set('modified_by', $uid);
    return $this;
  }

  /**
   * Gets the boolean indicating whether the user entity exists.
   *
   * @return boolean
   *   Boolean indicating whether the user entity exists.
   */
  public function getUserDeleted() {
    return $this->get('user_deleted')->value;
  }

  /**
   * Sets the boolean indicating whether the user entity exists.
   *
   * @param boolean $exists
   *   Boolean indicating whether the user entity exists.
   *
   * @return \Drupal\user_annex\Entity\UserAnnexInterface
   *   The called user_annex entity.
   */
  public function setUserDeleted($deleted) {
    $this->set('user_deleted', $deleted);
    return $this;
  }

  /**
   * Gets the User id from the modified user entity.
   *
   * @return int
   *   user id from the modified user entity.
   */
  public function getUserId() {
    return $this->get('user_id')->value;
  }

  /**
   * Sets the user id from the modified user entity.
   *
   * @param string $uid
   *   The User id from the modified user entity.
   *
   * @return \Drupal\user_annex\Entity\UserAnnexInterface
   *   The called user_annex entity.
   */
  public function setUserid($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * Gets the name from the modified user entity.
   *
   * @return string`
   *   The name from the modified user entity.
   */
  public function getUserName() {
    return $this->get('user_name')->value;
  }

  /**
   * Sets the name from the modified user entity.
   *
   * @param string $name
   *   The name from the modified user entity.
   *
   * @return \Drupal\user_annex\Entity\UserAnnexInterface
   *   The called user_annex entity.
   */
  public function setUserName($name) {
    $this->set('user_name', $name);
    return $this;
  }

  /**
   * Gets the status from the modified user entity.
   *
   * @return boolean
   *   The status from the modified user entity.
   */
  public function getUserStatus() {
    return $this->get('user_status')->value;
  }

  /**
   * Sets the status from the modified user entity.
   *
   * @param boolean $status
   *   The status from the modified user entity.
   *
   * @return \Drupal\user_annex\Entity\UserAnnexInterface
   *   The called user_annex entity.
   */
  public function setUserStatus($status) {
    $this->set('user_status', $status);
    return $this;
  }

  /**
   * Gets the difference between the modified user entity and a previous copy.
   *
   * @return string
   *   the difference between the modified user entity and a previous copy.
   */
  public function getDifference() {
    return $this->get('difference')->value;
  }

  /**
   * Sets the difference between the modified user entity and a previous copy.
   *
   * @param string $difference
   *   the differences between the modified user entity and a previous copy.
   *
   * @return \Drupal\user_annex\Entity\UserAnnexInterface
   *   The called user_history entity.
   */
  public function setDifference($difference) {
    if (strlen($difference) > 255) {
      $difference = substr($difference,0,252) . '...';
    }
    $this->set('difference', $difference);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    // Start with the base fields for a content entity type.
    // This includes only id for user_annex entities as specified
    // in the annotation above.
    $fields = parent::baseFieldDefinitions($entity_type);

    $weight = 0;

    // Define some fields specific to this entity type.
    $fields['label'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Label'))
      ->setDescription(t('Auto-create label for the user_annex entity'))
      ->setRequired(TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => $weight++,
      ])
    ->setDisplayOptions('form', [
      'type' => 'string_textfield',
      'weight' => $weight,
    ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Modified on'))
      ->setDescription(t('The time that the user_annex entity was created.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'timestamp',
        'weight' => $weight++,
      ])
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => $weight,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['action'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Action'))
      ->setDescription(t('The action used to modify the user entity.'))
      ->setDefaultValue('')
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => $weight++,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => $weight,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['modified_by'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Modified by'))
      ->setDescription(t('The account making the changes to the user entity.'))
      ->setSetting('target_type', 'user')
      ->setRequired(TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'entity_reference_label',
        'weight' => $weight++,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => $weight,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ]
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['user_deleted'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('User deleted'))
      ->setDescription(t('A boolean indicating whether the user entity was deleted.'))
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'boolean',
        'settings' => [
          'format' => 'yes-no',
        ],
        'weight' => $weight++,
      ])
    ->setDisplayOptions('form', [
      'type' => 'boolean_checkbox',
      'weight' => $weight,
    ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('User ID'))
      ->setDescription(t('The ID of the modified account.'))
      ->setSetting('target_type', 'user')
      ->setRequired(TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'entity_reference_entity_id',
        'weight' => $weight++,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
          ]
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['user_name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('The name of the modified account.'))
      ->setDefaultValue('')
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => $weight++,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => $weight,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['user_status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('User status'))
      ->setDescription(t('Whether the user is active or blocked.'))
      ->setDefaultValue(FALSE)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'boolean',
        'settings' => [
          'format' => 'custom',
          'format_custom_true' => 'Active',
          'format_custom_false' => 'Blocked',
        ],
        'weight' => $weight++,
      ])
      ->setDisplayOptions('form', [
        'type' => 'boolean',
        'weight' => $weight,
      ])
      ->setDisplayConfigurable('form', TRUE);

    $fields['difference'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Difference'))
      ->setDescription(t('The difference between this user_annex record and the previous one for same user account'))
      ->setDefaultValue('')
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'inline',
        'type' => 'string',
        'weight' => $weight++,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => $weight,
      ])
      ->setDisplayConfigurable('form', TRUE);

    return $fields;
  }

  // Dummy methods on missing fields
  public function isPublished() {
    return TRUE;
  }

  public function bundle() {
    return 'user_annex';
  }

}
